import importlib.resources
import pathlib
import re

import click
import jinja2

from canvas_course_tools.datatypes import GroupList, Student, StudentGroup

PARSE_STUDENT_RE = re.compile("(?P<name>.*) \((?P<id>.*)\) *(?:\[(?P<notes>.*)\])?")


@click.group()
def templates():
    """Generate files based on templates and group lists."""
    pass


@templates.command("render")
@click.argument("template")
@click.argument("group_list")
def render_template(template, group_list):
    """Render a template.

    The first argument, TEMPLATE, should be the name of one of the templates
    registered with the app. Use the `list` command to get a list of templates.

    The second argument, GROUP_LIST, should be a path to a group list file. If a
    file starts with a #-character, the rest of the line is interpreted as a
    title. If it starts with ##, the rest of the line is interpreted as a group
    name. There can be multiple groups defined in one file. All other non-empty
    lines are interpreted as a student name with optional student id and notes
    field. It must be of the form "student name (id) [notes]". For example, the
    following is a valid group list file:

        \b
        # Physics 101
        ## Group A
        Drew Ferrell (800057) [second year]
        Amanda James (379044)
        Antonio Morris (804407) [skips thursdays]

        \b
        ## Group B
        Elizabeth Allison (312702)
        James Morales (379332)

    \f
    Ignore the \b and \f characters in this docstring. They are to tell click to
    not wrap paragraphs (\b) and not display this note (\f).
    """
    file_contents = pathlib.Path(group_list).read_text()
    group_list = parse_group_list(file_contents)
    contents = render_template(template, group_list)
    click.echo(contents)


def parse_group_list(text):
    """Parse text and build a group list."""
    group_list = GroupList()
    current_group = StudentGroup()

    for line in text.splitlines():
        if not line:
            # empty line
            continue
        elif line.startswith("##"):
            # new group name
            group_name = line.removeprefix("##").strip()
            if current_group.students:
                # there are students in the current group, add them to the list
                group_list.groups.append(current_group)
            # create new group
            current_group = StudentGroup(name=group_name)
        elif line.startswith("#"):
            # group list name
            group_list.name = line.removeprefix("#").strip()
        else:
            # must be a student
            match = PARSE_STUDENT_RE.match(line)
            if match:
                current_group.students.append(
                    Student(name=match["name"], id=match["id"], notes=match["notes"])
                )
    group_list.groups.append(current_group)

    return group_list


def render_template(template_name, group_list: GroupList):
    """Render a template.

    Args:
        template_name (str): the name of the template
        group_list (GroupList): the group list as input for the template
    """
    env = jinja2.Environment(
        loader=jinja2.PackageLoader("canvas_course_tools", "templates"),
    )
    try:
        template = env.get_template(template_name, globals={"zip": zip})
    except jinja2.exceptions.TemplateNotFound:
        raise click.BadArgumentUsage(f"Template {template_name} not found!")
    return template.render(title=group_list.name, groups=group_list.groups)



import click

from canvas_course_tools import configfile
from canvas_course_tools.canvas_tasks import CanvasTasks


def get_canvas(server_alias):
    config = configfile.read_config()
    try:
        server = config["servers"][server_alias]
    except KeyError:
        raise click.UsageError(f"Unknown server '{server_alias}'.")
    else:
        return CanvasTasks(server["url"], server["token"])



import click

from canvas_course_tools import configfile
from canvas_course_tools.utils import get_canvas


@click.group()
def students():
    """Search for or list students."""
    pass


@students.command("list")
@click.argument("course_alias")
@click.option(
    "--all/-no-all",
    default=False,
    help="List all students without splitting them into sections.",
)
def list_students(course_alias, all):
    """List students in course COURSE_ALIAS.

    Students are split into sections by default.
    """
    config = configfile.read_config()
    server, course_id = (
        config["courses"][course_alias][k] for k in ("server", "course_id")
    )
    canvas = get_canvas(server)
    course = canvas.get_course(course_id)
    print(f"# {course.name}\n")
    if all:
        students = canvas.get_students(course_id)
        students.sort(key=lambda x: x.sortable_name)
        for student in students:
            print(f"{student.name} ({student.id})")
    else:
        for section in canvas.get_sections(course_id):
            print(f"## {section.name}\n")
            section.students.sort(key=lambda x: x.sortable_name)
            for student in section.students:
                print(f"{student.name} ({student.id})")
            print("\n")



from dataclasses import dataclass, field
from typing import Optional

from nameparser import HumanName


@dataclass
class Course:
    id: int
    name: str
    course_code: str
    term: str
    alias: Optional[str] = None


@dataclass
class Student:
    id: int
    name: str
    sortable_name: Optional[str] = None
    first_name: str = field(init=False)
    last_name: str = field(init=False)
    notes: Optional[str] = None

    def __post_init__(self):
        # Parse human names, allowing for underscores to group parts of the name
        name = HumanName(self.name)
        self.first_name = " ".join([name.first, name.middle]).rstrip()
        self.last_name = name.last.replace("_", " ")
        self.name = self.name.replace("_", " ")


@dataclass
class Section:
    id: int
    name: str
    students: list[Student]


@dataclass
class StudentGroup:
    name: str = ""
    students: list[Student] = field(default_factory=list)


@dataclass
class GroupList:
    name: str = ""
    groups: list[StudentGroup] = field(default_factory=list)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['canvas_course_tools']

package_data = \
{'': ['*'], 'canvas_course_tools': ['templates/*']}

install_requires = \
['appdirs>=1.4.4,<2.0.0',
 'canvasapi>=2.2.0,<3.0.0',
 'click>=8.1.3,<9.0.0',
 'jinja2>=3.1.2,<4.0.0',
 'nameparser>=1.1.1,<2.0.0',
 'python-dateutil>=2.8.1,<3.0.0',
 'rich-click>=1.5.2,<2.0.0',
 'rich>=12.5.1,<13.0.0',
 'tomli-w>=1.0.0,<2.0.0',
 'tomli>=2.0.1,<3.0.0']

entry_points = \
{'console_scripts': ['canvas = canvas_course_tools.cli:cli']}

setup_kwargs = {
    'name': 'canvas-course-tools',
    'version': '0.1.0',
    'description': 'Canvas course tools',
    'long_description': 'None',
    'author': 'David Fokkema',
    'author_email': 'd.b.r.a.fokkema@vu.nl',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)



from canvasapi import Canvas
from canvasapi.exceptions import Forbidden, InvalidAccessToken, ResourceDoesNotExist

from canvas_course_tools.datatypes import Course, Section, Student


class CanvasTasks:

    """Collection of canvas-related tasks."""

    def __init__(self, url, token):
        """Initialize the class.

        Args:
            url: a string containing the Canvas server URL
            token: a string containing the Canvas API access token
        """
        self.canvas = Canvas(url, token)

    def list_courses(self):
        """List Canvas courses.

        Returns:
            A list of Canvas course objects.
        """
        courses = self.canvas.get_courses(include=["term"])
        return [
            Course(
                id=course.id,
                name=course.name,
                course_code=course.course_code,
                term=course.term["name"],
            )
            for course in courses
        ]

    def get_course(self, course_id):
        """Get a Canvas course by id.

        Args:
            course_id: a Canvas course id.

        Returns:
            A Canvas course object.
        """
        course = self.canvas.get_course(course_id, include=["term"])
        return Course(
            id=course.id,
            name=course.name,
            course_code=course.course_code,
            term=course.term["name"],
        )

    def get_students(self, course_id):
        """Get all students in a course.

        Args:
            course_id (integer): the Canvas course id

        Returns:
            list: a list of student objects
        """
        course = self.canvas.get_course(course_id)
        students = course.get_users(enrollment_type=["student"])
        return [
            Student(
                id=student.id,
                name=student.short_name,
                sortable_name=student.sortable_name,
            )
            for student in students
        ]

    def get_sections(self, course_id):
        """Get a list of sections, including students

        Args:
            course_id (int): the course id

        Returns:
            List[Section]: A list of Sections
        """
        course = self.canvas.get_course(course_id)
        sections = course.get_sections(include=["students"])
        return [
            Section(
                id=section.id,
                name=section.name,
                students=[
                    Student(
                        id=student["id"],
                        name=student["short_name"],
                        sortable_name=student["sortable_name"],
                    )
                    for student in section.students
                ],
            )
            for section in sections
            if section.students is not None
        ]
